%

%
% Copyright (c) 2015 LAAS/CNRS
% All rights reserved.
%
% Redistribution  and  use  in  source  and binary  forms,  with  or  without
% modification, are permitted provided that the following conditions are met:
%
%   1. Redistributions of  source  code must retain the  above copyright
%      notice and this list of conditions.
%   2. Redistributions in binary form must reproduce the above copyright
%      notice and  this list of  conditions in the  documentation and/or
%      other materials provided with the distribution.
%
% THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
% WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
% MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
% ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
% WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
% IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
%
%                                           Anthony Mallet on Mon Jun  8 2015
%
classdef component < dynamicprops & handle
  properties (SetAccess = private)
    meta
  end

  methods
    function self = component(handle, client, meta)
      % genomix.component - Remote component control interface
      %
      self.meta = meta;

      svcs = fieldnames(meta.services);
      for i = 1:numel(svcs)
        n = regexprep(svcs{i}, '(^[0-9]|[^0-9a-zA-Z_])', '_');
        h = self.addprop(n);
        h.SetAccess = 'private';

        if isfield(meta.services.(svcs{i}), 'meta')
          defarg = self.meta2defarg(meta.services.(svcs{i}).meta);
        else
          defarg = struct();
        end

        args.id = handle.id;
        args.client = client;
        args.name = svcs{i};
        args.meta = meta.services.(svcs{i});
        args.defarg = defarg;
        self.(n) = @(varargin) handle.rqst(args, varargin{:});
      end

      ports = fieldnames(meta.ports);
      for i = 1:numel(ports)
        n = regexprep(ports{i}, '(^[0-9]|[^0-9a-zA-Z_])', '_');
        h = self.addprop(n);
        h.SetAccess = 'private';

        args.id = handle.id;
        args.client = client;
        args.name = ports{i};
        args.meta = meta.ports.(ports{i});

        if meta.ports.(ports{i}).simple
          self.(n) = @() handle.port(args);
        else
          self.(n) = @(name) handle.port(args, name);
        end
      end
    end
  end


  methods (Static, Access = private)
    % --- meta2defarg ------------------------------------------------------
    %
    function defarg = meta2defarg(meta)
      if isstruct(meta)
        defarg = struct();
        for p = fieldnames(meta)'
          if isfield(meta.(p{1}), 'default')
            defarg.(p{1}) = meta.(p{1}).default;
          elseif isfield(meta.(p{1}), 'items')
            defarg.(p{1}) = genomix.component.meta2defarg(meta.(p{1}).items);
          end
        end
      else
        defarg = {};
        for k = 1:length(meta)
          if isfield(meta{k}, 'default')
            defarg(k) = { meta{k}.default };
          elseif isfield(meta{k}, 'items')
            defarg(k) = { genomix.component.meta2defarg(meta{k}.items) };
          end
        end
      end
    end
  end
end
